/* hilite_compromise.pov
 * Persistence of Vision Ray Tracer scene description file
 *
 * This program is an attempt to find reasonable compromise specular highlight
 * values for glossy non-metallic objects, since POV-Ray versions 3.7.0 and
 * earlier do not support Fresnel highlights.
 *
 * The number to the upper left of each test object is the index of refraction
 * of the left half.  The number to the upper right is the constant reflection
 * value of the right half, and is intended to be used as a 'specular albedo'
 * value in POV-Ray 3.7.
 *
 * If you are using POV-Ray 3.6.2 or earlier, you will need to multiply the
 * upper right number by the following formula to use as a 'specular' value:
 *
 *   (1 / roughness + 2) / (4 * (2 - pow (2, -0.5 / roughness)))
 *
 * If you are using POV-Ray 3.8 or later, you should use finish-level Fresnel
 * instead of consulting this program.
 *
 * Copyright © 2014, 2020 Richard Callwood III.  Some rights reserved.
 * This work is licensed under the GNU General Public License 3.0.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-3.0.html for the legalese.
 *
 * Vers  Date         Comments
 * ----  ----         --------
 * 1.0   2014-Nov-03  Created.
 * 2.0   2020-Jan-30  The formula is extended to cover low IORs, and test cases
 *                    are added from IOR 1.05 to 1.30.
 */
#version 3.7;

global_settings { assumed_gamma 1 charset utf8 }

background { rgb <0.5, 0.7, 1.0> }

#declare NROWS = 3;
#declare NCOLS = 6;
#declare SPACE = 0.3;
#declare ROOM = 0.6;
#declare S_FONT = "arial" // a Unicode font with mathematical symbols

#declare WIDTH = SPACE + NCOLS * (2 + SPACE);
#declare HEIGHT = SPACE + NROWS * (2 + ROOM);

#declare hSuggest = floor (image_width * HEIGHT / WIDTH + 0.5);
#if (floor (image_height + 0.5) != hSuggest)
  #warning concat
  ( "Suggested image dimensions: +W", str (image_width, 0, 0),
    " +H", str (hSuggest, 0, 0)
  )
#end

camera
{ orthographic
  location <WIDTH/2, HEIGHT/2, -10>
  right WIDTH * x
  up HEIGHT * y
}

// This curve fit breaks down at IORs above 3 or so,
// but it's good enough for most common materials:
#declare fn_Fudge = function (x)
{ select (x - 1.4, pow (x - 1, 1.5) * 0.1415, x * 0.142 - 0.163)
}

#macro Test (IofR)
  #local Fudge = fn_Fudge (IofR);
  union
  { cylinder
    { -x, 0, 1
      pigment { rgb 0 }
      finish { reflection { 0 1 fresnel } conserve_energy }
      interior { ior IofR }
    }
    cylinder
    { x, 0, 1
      pigment { rgb 0 }
      finish { reflection { Fudge } }
    }
    text
    { ttf S_FONT concat (str (IofR, 0, 2), " \u2192 ", str (Fudge, 0, 4)) 1, 0
      pigment { red 1 }
      finish { ambient 0.5 }
      scale 0.3
      translate <-1, 1.1, 0>
    }
    no_reflection
  }
#end

#declare fn_Select_IOR = function (x)
{ select (x - 6, 1.05 + x * 0.05, 0.75 + x * 0.1)
}

#for (Row, 0, NROWS-1)
  #for (Col, 0, NCOLS-1)
    object
    { Test (fn_Select_IOR (Row * NCOLS + Col))
      translate
      < SPACE + 1 + Col * (2 + SPACE),
        SPACE + 1 + (NROWS - 1 - Row) * (2 + ROOM),
        0
      >
    }
  #end
#end
